//=============================================================================
//
//  Code written for Arduino Uno
//
//  CRYSTALFONTZ CFAG14432D 144x32 Graphic LCD in SPI mode
//
//  ref: https://www.crystalfontz.com/search.php?q=cfag14432d
//
//  video: https://www.youtube.com/watch?v=By0t1mrXsO8
//
//  tutorial: https://forum.crystalfontz.com/showthread.php/7395
//
//  2016 - 06 - 09 Brent A. Crosby
//
//=============================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//=============================================================================
#include "lcd_draw.h"
#include "CF_Logo.h"
//=============================================================================
#define FAST_SPI 1

//FAST_SPI = 0: The SS pin is high for ~429uS (24-bits)
//              The clock frequency is ~57KHz
//              Full graphic update takes ~285mS
//FAST_SPI = 1: With no delays, the SS pin is high for ~12uS (24-bits)
//              The clock frequency is ~2MHz
//              But that is too fast for the ST7920, so slow it down
//              with the additional _delay_us(1) calls, which makes:
//              The SS pin is high for ~39uS (24-bits)
//              The clock frequency is ~615KHz
//              Full graphic update takes ~22.5mS

//Software SPI (10-bit transfers, difficult to do using the hardware SPI)
#define SPIPORT (PORTB)
#define SPITOGGLE (PINB)
// PB5 (0x20) is SCK  (output) green  OLED pin 12
#define SPI_SCK_PIN (13)
#define SCK_MASK (0x20)
#define CLR_SCK (PORTB &= ~(SCK_MASK))
#define SET_SCK (PORTB |=  (SCK_MASK))
// PB4 (0x10) is MISO (input)  blue   OLED pin 13
//(reference only, it is an input)
#define SPI_MISO_PIN (12)
#define MISO_MASK (0x10)
#define CLR_MISO (PORTB &= ~(MISO_MASK))
#define SET_MISO (PORTB |=  (MISO_MASK))
// PB3 (0x08) is MOSI (output) violet OLED pin 14
#define SPI_MOSI_PIN (11)
#define MOSI_MASK (0x08)
#define CLR_MOSI (PORTB &= ~(MOSI_MASK))
#define SET_MOSI (PORTB |=  (MOSI_MASK))
// DB2 (0x04) is SS   (output) gray   OLED pin 15
#define SPI_SS_PIN (10)
#define SS_MASK (0x04)
#define CLR_SS  (PORTB &= ~(SS_MASK))
#define SET_SS  (PORTB |=  (SS_MASK))
//=============================================================================
// Graphic data to display on LCD.
// Image2Code may be helpful to create your own screen image:
//    https://forum.crystalfontz.com/showthread.php/5854
//    http://sourceforge.net/p/image2code/code/HEAD/tree/
// screen used for the CFAG14432b photos on the website
const uint8_t cfag14432b_Logo[32][18] PROGMEM = {
     {0x00,0x38,0x0E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0x00,0x6C,0x1B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0x00,0xC6,0x31,0x80,0x03,0xE0,0x00,0x00,0x00,0x00,0x40,0x00,0x70,0xF0,0x00,0x00,0x02,0x00},
     {0x01,0x82,0x60,0xC0,0x07,0xF8,0x00,0x00,0x00,0x00,0xC0,0x00,0x71,0xF0,0x00,0x00,0x06,0x00},
     {0x03,0x00,0xE0,0x60,0x0F,0xF8,0x00,0x00,0x00,0x01,0xC0,0x00,0xE3,0x80,0x00,0x00,0x0E,0x00},
     {0x07,0x01,0xE0,0x70,0x1E,0x3C,0x77,0xE1,0xC7,0xE7,0xE3,0xF0,0xEF,0xE7,0xE0,0xEF,0x3F,0xFF},
     {0x0F,0x03,0xE0,0x78,0x1C,0x1C,0x7E,0xE1,0x8F,0xF7,0xE7,0xF8,0xEF,0xEF,0xF0,0xFF,0xBF,0xFF},
     {0x1F,0x03,0xE0,0x7C,0x3C,0x00,0x78,0xE3,0x9C,0x73,0x8E,0x38,0xE3,0x9E,0x78,0xF3,0x9C,0x03},
     {0x00,0x03,0xE0,0x00,0x38,0x00,0xF0,0xE3,0x1C,0x03,0x80,0x38,0xE7,0x1C,0x39,0xE3,0x9C,0x07},
     {0xFF,0xE3,0xEF,0xFF,0xB8,0x00,0xE0,0xE7,0x1F,0x83,0x83,0xF9,0xC7,0x38,0x39,0xC7,0x1C,0x0E},
     {0xFF,0xE3,0xEF,0xFF,0xB8,0x00,0xE0,0xE6,0x0F,0xC3,0x8F,0xF9,0xC7,0x38,0x39,0xC7,0x1C,0x1C},
     {0xFF,0xE3,0xE0,0x00,0x38,0x38,0xE0,0x6E,0x03,0xE7,0x1E,0x71,0xC7,0x38,0x39,0xC7,0x38,0x38},
     {0xFF,0xE3,0xEF,0xFF,0xBC,0x78,0xE0,0x6C,0x00,0xE7,0x1C,0x71,0xC7,0x38,0x71,0xC7,0x38,0x70},
     {0xFF,0xE3,0xEF,0xFF,0x9F,0xF1,0xC0,0x7C,0x38,0xE7,0x1C,0xF1,0xCE,0x3C,0xF3,0x8E,0x38,0xE0},
     {0xFF,0xE3,0xE0,0x00,0x1F,0xE1,0xC0,0x78,0x3F,0xC7,0xDF,0xF3,0x8E,0x1F,0xE3,0x8E,0x3F,0xFF},
     {0xFF,0xE3,0xEF,0xFF,0x87,0x81,0xC0,0x78,0x1F,0x83,0xCF,0x73,0x8E,0x0F,0xC3,0x8E,0x1F,0xFF},
     {0xFF,0xE3,0xEF,0xFF,0x80,0x00,0x00,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xE3,0xE0,0x00,0x00,0x00,0x00,0xF0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xE3,0xEF,0xFF,0x80,0x00,0x03,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xE3,0xEF,0xFF,0x80,0x00,0x03,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xE3,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xE3,0xEF,0xFF,0x80,0x73,0xE2,0x07,0x04,0x08,0x23,0x8E,0x7C,0x0F,0x00,0x08,0x00,0x00},
     {0xFF,0xE3,0xEF,0xFF,0x80,0x8A,0x05,0x08,0x8C,0x18,0x64,0x51,0x42,0x10,0x80,0x00,0x00,0x00},
     {0x00,0x03,0xE0,0x00,0x01,0x02,0x05,0x10,0x54,0x28,0xA0,0x41,0x42,0x10,0x1C,0xA9,0xC7,0x00},
     {0x1F,0x03,0xE0,0x7C,0x01,0x03,0xC5,0x10,0x04,0x28,0xA1,0x81,0x7E,0x0C,0x22,0xCA,0x28,0x80},
     {0x0F,0x03,0xE0,0x78,0x01,0x02,0x08,0x91,0xC4,0x49,0x20,0x42,0x42,0x03,0x3E,0x8B,0xE6,0x00},
     {0x07,0x01,0xE0,0x70,0x01,0x02,0x0F,0x90,0x44,0x7D,0xF0,0x44,0x42,0x00,0xA0,0x8A,0x01,0x00},
     {0x03,0x00,0xE0,0x60,0x00,0x8A,0x10,0x48,0x84,0x08,0x24,0x48,0x42,0x10,0xA2,0x8A,0x28,0x80},
     {0x01,0x82,0x60,0xC0,0x00,0x72,0x10,0x47,0x04,0x08,0x23,0x9F,0x7C,0x0F,0x1C,0x89,0xC7,0x00},
     {0x00,0xC6,0x31,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0x00,0x6C,0x1B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0x00,0x38,0x0E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}
     };
const uint8_t v_stripes[32][18] PROGMEM = {
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA}
     };
const uint8_t h_stripes[32][18] PROGMEM = {
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
     {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
     {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}
     };
const uint8_t checkerboard[32][18] PROGMEM = {
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
     {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
     {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55}
     };
//-----------------------------------------------------------------------------
const uint8_t random_walk_data[200] PROGMEM = 
  {133,128,126,132,133,128,128,123,126,133,133,133,140,138,147,148,156,164,173,168,
   170,166,162,167,167,169,173,180,181,176,178,182,182,179,184,189,196,205,208,210,
   219,224,222,227,233,235,243,248,245,254,255,252,247,243,240,237,230,229,222,214,
   209,207,212,211,202,198,201,192,194,193,196,193,198,197,198,193,185,184,182,181,
   182,178,173,171,170,164,157,150,141,143,143,134,131,124,124,125,130,123,120,121,
   120,120,120,122,118,109,107,112,113,105,102,106, 97, 91, 90, 89, 94, 87, 92, 90,
    88, 88, 83, 74, 66, 62, 59, 51, 42, 37, 39, 39, 31, 31, 24, 28, 22, 23, 16, 17,
    13, 16, 18, 12, 13,  5, 10,  3,  8,  0,  2,  6, 12, 19, 19, 27, 33, 35, 40, 36,
    35, 42, 44, 53, 55, 51, 59, 58, 61, 62, 70, 78, 76, 80, 87, 94, 95, 93, 95,104,
   108,112,113,109,111,117,117,116,123,127,122,120,128,127,124,126,121,122,129,136};
//=============================================================================
void SPI_WriteByte(uint8_t data)
  {
#if FAST_SPI
  //Pretty fast software SPI 8-bit transfer code
  //Several ideas taken from the fastest non-assembly implementation at:
  //http://nerdralph.blogspot.com/2015/03/fastest-avr-software-spi-in-west.html
  //
  //Pre-calculate the value that will drive clk and data low in one cycle.
  //(Note that this is not interrupt safe if an ISR is writing to the same port)
  register uint8_t
    force_clk_and_data_low;

  //Pre-calculate the value that will drive clk and data low in one
  //operation.
  force_clk_and_data_low = (SPIPORT & ~(MOSI_MASK | SCK_MASK) );

  //Now clock the 8 bits of data out. It needs the delays.
  SPIPORT = force_clk_and_data_low;
  if(data & 0x80)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x40)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x20)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x10)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x08)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x04)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x02)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(2);
  SPIPORT = force_clk_and_data_low;
  if(data & 0x01)
    SPITOGGLE = MOSI_MASK;
  _delay_us(1);
  SPITOGGLE = SCK_MASK;
  _delay_us(1);

#else
  //Straight-forward software SPI 8-bit transfer code, perhaps
  //easier to understand, possibly more portable. Certainly slower.

  //(bits 0-7) Push each of the 8 data bits out.
  for(uint8_t mask=0x80;mask;mask>>=1)
    {
    //Set the MOSI pin high or low depending on if our mask
    //corresponds to a 1 or 0 in the data.
    if(mask&data)
      {
      digitalWrite(SPI_MOSI_PIN, HIGH);
      }
    else
      {
      digitalWrite(SPI_MOSI_PIN, LOW);
      }
    //Clock it in.
    digitalWrite(SPI_SCK_PIN, HIGH);
    digitalWrite(SPI_SCK_PIN, LOW);
    }
#endif
  }
//-----------------------------------------------------------------------------
void write_command(uint8_t command)
  {
  //Select the chip, starting a 24-bit SPI transfer
#if FAST_SPI
  SET_SS;
#else
  digitalWrite(SPI_SS_PIN, HIGH);
#endif
  SPI_WriteByte(0xF8);  // send command header
  SPI_WriteByte(command&0xF0);  // send high nibble
  SPI_WriteByte((command<<4)&0xF0);  // send low nibble
  //Release the chip, ending the SPI transfer
#if FAST_SPI
  CLR_SS;
#else
  digitalWrite(SPI_SS_PIN, LOW);
#endif
  }
//-----------------------------------------------------------------------------
void write_data(uint8_t data)
  {
  //Select the chip, starting a 24-bit SPI transfer
#if FAST_SPI
  SET_SS;
#else
  digitalWrite(SPI_SS_PIN, HIGH);
#endif
  SPI_WriteByte(0xFA);  // send data header
  SPI_WriteByte(data&0xF0);  // send high nibble
  SPI_WriteByte((data<<4)&0xF0);  // send low nibble
  //Release the chip, ending the SPI transfer
#if FAST_SPI
  CLR_SS;
#else
  digitalWrite(SPI_SS_PIN, LOW);
#endif
  }
//-----------------------------------------------------------------------------
//X is only adressible to the word, a grouping of 16 horizontal pixels:
//  0-17 => 18 * 8 = 144
void Set_Graphics_Mode_Address(uint8_t x_word, uint8_t y)
  {
  // Access extended functions register, graphics mode
  write_command(0x34);
  //Set Y pixel address
  write_command(0x80 | y);
  //Set X byte address
  write_command(0x80 | x_word);
  // Exit extended functions register, stay in graphics mode
  write_command(0x36);
  }
//-----------------------------------------------------------------------------
void show_text_demo(void)
  {
  static const uint8_t displaytext[2][18] PROGMEM =
    {{'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','G','H'},
     {'H','a','r','d','w','a','r','e',' ','C','G','R','O','M',' ', 1 , 2 ,'!'}};
  
  register uint8_t
    page_address;
  register uint8_t
    column_address;

  write_command(0x38);  // function set -- 8 bit mode, basic instruction set
  write_command(0x0F);  // display control -- cursor on, blinking
  write_command(0x01);  // display clear
  _delay_ms(2);
  write_command(0x06);  // entry mode -- cursor moves right, address counter increased by 1

  for (page_address = 0; page_address < 2; page_address++)
    {
    if(page_address==1)
      {
      write_command(0x90);
      }
    for (column_address = 0; column_address < 18; column_address++)
      {
      write_data(pgm_read_byte(&displaytext[page_address][column_address]));
      }
    }
  }
//-----------------------------------------------------------------------------
void Fill_FrameBuffer(uint8_t fill_data)
  {
  register uint8_t
    i;
  register uint8_t
    j;

  //Set the FrameBuffer to a given data.
  for(j=0;j<=31;j++)
    {
    for(i=0;i<=17;i++)
      {
      framebuffer[j][i]=fill_data;
      }
    }
  }
//-----------------------------------------------------------------------------
// ref: http://stackoverflow.com/questions/28787177/c-arduino-passing-pointer-to-2d-array-stored-in-progmem-to-a-function
void Copy_PROGMEM_Image_To_FrameBuffer(const uint8_t (&screen)[32][18],uint8_t invert)
  {
  register uint8_t
    i;
  register uint8_t
    j;

  //Get the image from the flash, stuff it into the RAM frame buffer.
  for(j=0;j<=31;j++)
    {
    //Dump the data out for this line.
    if(invert)
      {
      for(i=0;i<=17;i++)
        {
        //Inverted
        framebuffer[j][i]=~(pgm_read_byte(&screen[j][i]));
        }
      }
    else
      {
      for(i=0;i<=17;i++)
        {
        //Normal
        framebuffer[j][i]=pgm_read_byte(&screen[j][i]);
        }
      }
    }
  }
//-----------------------------------------------------------------------------
void Send_FrameBuffer_To_LCD(void)
  {
  register uint8_t
    i;
  register uint8_t
    j;
  write_command(0x06);  // entry mode -- cursor moves right, address counter increased by 1
  write_command(0x34);  // extended function
  write_command(0x36);  // graphic mode on

  //Send the FrameBuffer RAM data to the LCD.
  for(j=0;j<=31;j++)
    {
    //Set Y address to this line, reset X address
    Set_Graphics_Mode_Address(0,j);
    for(i=0;i<=17;i++)
      {
      write_data(framebuffer[j][i]);
      }
    }
  }
//=============================================================================
// Slide the logo around.
void bitmap_demo(void)
  {
  uint8_t
    i;
  uint8_t
    j;
  uint8_t
    dir;    
  j=0;
  dir=1;
  for(i=0;i<=60;i++)
    {
    Fill_FrameBuffer(0x00);
    Draw_Bitmap(       i,   j,23,21,(const uint8_t *)CF_Logo_23x21);
    Draw_Bitmap(143-23-i,11-j,23,21,(const uint8_t *)CF_Logo_23x21);
    Send_FrameBuffer_To_LCD();

    if(dir)
      {
      //up  
      if(j<11)
        {
        j++;
        }
      else
        {
        dir=0;
        j=10;
        }
      }
    else
      {
      //down  
      if(0<j)
        {
        j--;
        }
      else
        {
        dir=1;
        j=1;
        }
      }
    _delay_ms(25);
    }
  }
//=============================================================================
// Bar graph / strip chart demo
void graph_demo(uint16_t count)
  {
  static const char max_string[5]   PROGMEM ="mx:";
  static const char min_string[5]   PROGMEM ="mn:";
  static const char value_string[5] PROGMEM ="Val:";
  char
    temp_string[10];
    
  uint16_t
    base_index;
  base_index=0;

  #define GRAPHSTART (45)
  #define GRAPHWIDTH (68)

  uint8_t
    initial_prev_y;
  //Calculate the first point of the next graph, so we have a starting point.
  initial_prev_y=(255-pgm_read_byte(&random_walk_data[0]))>>3;

  while(count--)
    {
    uint8_t
      max_data,max_y;
    uint8_t
      min_data,min_y;

    Fill_FrameBuffer(0x00);
    // horizontal axis
    horizontal_line(GRAPHSTART,15,GRAPHSTART+GRAPHWIDTH);

    uint8_t
      prev_y;
    uint8_t
      cur_data;

    prev_y=initial_prev_y;

    max_data=0;
    min_data=255;
    for(uint16_t x=0;x<=GRAPHWIDTH;x++)
      {
      uint8_t
        data;
      uint8_t
        this_y;
      data=pgm_read_byte(&random_walk_data[(base_index+x)%200]);

      //Map data from 0-255 to 0-31 (divide by 8)
      this_y=(255-data)>>3;

      //Remember the min and max for the tic marks and text.
      if(data<min_data)
        {
        min_data=data;
        min_y=this_y;
        }
      if(max_data<data)
        {
        max_data=data;
        max_y=this_y;
        }
      //Draw from the last value to this value, so the line
      //does not have gaps.    
      if(prev_y<this_y)
        {
        // up line
        vertical_line(x+GRAPHSTART,prev_y,this_y);
        }
      else  
        {
        // down line
        vertical_line(x+GRAPHSTART,this_y,prev_y);
        }
      prev_y=this_y;

      //Remember the start Y for next pass, and the Y
      //for the text value.
      if(x==0)
        {
        initial_prev_y=this_y;
        cur_data=data;
        }
      }
    if(base_index==0)
      {
      base_index=200;
      }
    else
      {
      base_index--;
      }

    //Small text of max & min
    put_string_P(143-24, 0,4,max_string);
    put_string_P(143-24,15,4,min_string);
    dec_3_to_string(max_data,temp_string);
    put_string(  143-24, 8,4,temp_string);
    dec_3_to_string(min_data,temp_string);
    put_string(  143-24,23,4,temp_string);

    //Min & Max tics
    horizontal_line(GRAPHSTART+GRAPHWIDTH+2,min_y,GRAPHSTART+GRAPHWIDTH+4);
    horizontal_line(GRAPHSTART+GRAPHWIDTH+2,max_y,GRAPHSTART+GRAPHWIDTH+4);

    //Big text of current value.
    put_string_P(0,0,4,value_string);
    dec_3_to_string(cur_data,temp_string);
    put_big_number(0,10,3,temp_string);

    //Current value bar graph
    outline_rectangle(GRAPHSTART-8,initial_prev_y,GRAPHSTART-2,31);
    outline_rectangle(GRAPHSTART-6,initial_prev_y,GRAPHSTART-4,31);

    //That is a complete frame. Send it to the hardware.
    Send_FrameBuffer_To_LCD();
    _delay_ms(10);
    }
  }
//=============================================================================
void Init_LCD()
  {
  write_command(0x30);    // standard function set
  write_command(0x0C);    // cursor on, not blinking
  write_command(0x01);    // clear text screen
  _delay_ms(2);

  //clear the framebuffer send to the graphics screen
  Fill_FrameBuffer(0x00);
  Send_FrameBuffer_To_LCD();
  }
//=============================================================================
void setup()
  {
  //General setup, port directions.
  // PB5 (0x20) is SCK  (output) green  OLED pin 12
  pinMode(SPI_SCK_PIN, OUTPUT);
  // PB4 (0x10) is MISO (input)  blue   OLED pin 13
  //(reference only, it is an input)
  pinMode(SPI_MISO_PIN, INPUT);
  // PB3 (0x08) is MOSI (output) violet OLED pin 14
  pinMode(SPI_MOSI_PIN, OUTPUT);
  // DB2 (0x04) is SS   (output) gray   OLED pin 16
  pinMode(SPI_SS_PIN, OUTPUT);

  //SPI Clock is idle low  (SPI data is don't care)
  digitalWrite(SPI_SCK_PIN, LOW);
  //device not selected
  digitalWrite(SPI_SS_PIN, LOW);

  Init_LCD();
  }
//===========================================================================
void loop()
  {
  // Initialize the display
  Init_LCD();

  //Slide the CF logo happily around 
  bitmap_demo();

  //Push some static strings up
  Copy_PROGMEM_Image_To_FrameBuffer(cfag14432b_Logo,0);
  Send_FrameBuffer_To_LCD();
  _delay_ms(1500);

  Copy_PROGMEM_Image_To_FrameBuffer(v_stripes,0);
  Send_FrameBuffer_To_LCD();
  _delay_ms(750);

  Copy_PROGMEM_Image_To_FrameBuffer(v_stripes,1);
  Send_FrameBuffer_To_LCD();
  _delay_ms(750);

  Copy_PROGMEM_Image_To_FrameBuffer(h_stripes,0);
  Send_FrameBuffer_To_LCD();
  _delay_ms(750);

  Copy_PROGMEM_Image_To_FrameBuffer(h_stripes,1);
  Send_FrameBuffer_To_LCD();
  _delay_ms(750);

  Copy_PROGMEM_Image_To_FrameBuffer(checkerboard,0);
  Send_FrameBuffer_To_LCD();
  _delay_ms(750);

  Copy_PROGMEM_Image_To_FrameBuffer(checkerboard,1);
  Send_FrameBuffer_To_LCD();
  _delay_ms(750);

  //Show a graph , strip chart, text etc
  graph_demo(400);

  //Demo hardware CGROM
  show_text_demo();
  _delay_ms(2000);
  //Still in txt mode, loop will put us back in gfx mode
  }
//===========================================================================




